WATTBIKE_HUB_LOGIN_URL = 'https://api.wattbike.com/v2/login'
WATTBIKE_HUB_RIDESESSION_URL = 'https://api.wattbike.com/v2/classes/RideSession'
WATTBIKE_HUB_FILES_BASE_URL = \
    'https://api.wattbike.com/v2/files/{user_id}_{session_id}.{extension}'
WATTBIKE_HUB_USER_URL = 'https://api.wattbike.com/v2/classes/_User'
